@@ -23,7 +23,6 @@ import getpass
 import requests
 import urllib2
 import datetime
-import calendar
 import tempfile
 import stat
 import platform
@@ -101,6 +100,8 @@ class CLI(object):
         elif self.global_config.get('apikey'):
             access_key, secret = self.global_config.get('apikey').split(':')
             self.client.authenticator = BasicAuth(access_key, secret)
+            if getattr(self, 'user', None):
+                self.user.authenticator = self.client.authenticator
 
     def pre_refresh_token(self, req):
         self.info('Refreshing OAuth2 token...')
@@ -147,7 +148,7 @@ class CLI(object):
                 raise
             return 1
         except Exception as e:
-            message = u'An unexpected error has occured: {exc}.\n'.format(exc=e)
+            message = u'An unexpected error has occurred: {exc}.\n'.format(exc=e)
             if global_endpoint_info:
                 message += ('The remote server handling the last request was '
                             '{remotehost}:{remoteport}.\n'
@@ -274,7 +275,7 @@ class CLI(object):
 
     def prompt(self, prompt, noecho=False):
         method = getpass.getpass if noecho else raw_input
-        input = method(prompt + ': ')
+        input = method(prompt + ': '.encode('ascii'))
         return input
 
     def confirm(self, prompt, default='n'):
@@ -365,7 +366,7 @@ class CLI(object):
                 'key': CLIENT_KEY, 'secret': CLIENT_SECRET}
             try:
                 token = self.authorize_client(urlmap.get('token'), credential, username, password)
-            except Exception as e:
+            except Exception:
                 self.die('Username and password do not match. Try again.')
             token['url'] = credential['token_url']
             config = GlobalConfig()
@@ -435,6 +436,87 @@ class CLI(object):
         if self.confirm('Connect the current directory to "{0}"?'.format(args.application), 'y'):
             self._connect(args)
 
+    def cmd_traffic(self, args):
+
+        duration = self.parse_duration(args.duration)
+        self.info('Retrieving traffic metrics on {0} for the {1}'.format(args.application, duration))
+        url = '/applications/{0}/metrics/http?range={1}'.format(args.application, args.duration)
+
+        # dictionary of the array that is returned by the service
+        # [0] 1373387184, # timestamp
+        # [1] 4.808187, # 2xxs req/sec
+        # [2] 0, # 3xxs req/sec
+        # [3] 3.4046783, # 4xxs req/sec
+        # [4] 0, # 5xxs req/sec
+        # [5] 0.6730711, # application latency
+        # [6] 0.0046018595 # platform latency
+
+
+        try:
+            res = self.user.get(url)
+            services_table = [
+                [
+                    'time',
+                    '2xx',
+                    '3xx',
+                    '4xx',
+                    '5xx',
+                    'total req/sec',
+                    'platform latency',
+                    'application latency',
+                    'total latency'
+                ]
+            ]
+            for metric in res.items:
+                services_table.append([
+                    time.strftime("%a, %d %b %Y %H:%M:%S", time.gmtime(metric[0])),
+                    int(metric[1]),
+                    int(metric[2]),
+                    int(metric[3]),
+                    int(metric[4]),
+                    int(metric[1]+metric[2]+metric[3]+metric[4]),
+                    '{0} ms'.format(int(metric[6]*1000)),
+                    '{0} ms'.format(int(metric[5]*1000)),
+                    '{0} ms'.format(int(metric[6]*1000 + metric[5]*1000))
+                ])
+            pprint_table(services_table)
+        except RESTAPIError as e:
+            self.die('Retrieving traffic metrics failed: {1}'.format(args.application, e))
+
+    def cmd_memory(self, args):
+
+        duration = self.parse_duration(args.duration)
+        service_name, instance_id = self.parse_service_instance(args.service_or_instance, args.cmd)
+        self.info('Retrieving memory metrics on {0}.{1} for the {2}'.format(service_name, instance_id, duration))
+        url = '/applications/{0}/services/{1}/instances/{2}/metrics/memory?range={3}' \
+            .format(args.application, service_name, instance_id, args.duration)
+
+        # dictionary of the array that is returned by the service
+        # [0] 1376073600, # timestamp
+        # [1] 4.2341864E7, # inactive + active page size + resident state size (total used)
+        # [2] 1.34217728E8, # memory size at time
+        # [3] 1861094.4,
+        # [4] 49.1875864E7, # unused memory
+        # [5] 4.0493056E7, # resident set size
+        # [6] 339968,
+        # [7] 240588.8, # active page cache
+        # [8] 1268249.6 # inactive page cache
+
+        try:
+            res = self.user.get(url)
+            services_table = [ ['time', 'overage', 'unused', 'used', 'allocated'] ]
+            for metric in res.items:
+                services_table.append([
+                    time.strftime("%a, %d %b %Y %H:%M:%S", time.gmtime(metric[0])),
+                    bytes2human(abs(metric[2]-metric[1])) if metric[1] > metric[2] else bytes2human(0),
+                    bytes2human(metric[4]),
+                    bytes2human(metric[1]),
+                    bytes2human(metric[2]),
+                ])
+            pprint_table(services_table)
+        except RESTAPIError as e:
+            self.die('Retrieving memory metrics failed: {1}'.format(args.application, e))
+
     def cmd_connect(self, args):
         url = '/applications/{0}'.format(args.application)
         try:
@@ -464,7 +546,7 @@ class CLI(object):
             return
         self.info('Destroying "{0}"'.format(to_destroy))
         try:
-            res = self.user.delete(url)
+            self.user.delete(url)
         except RESTAPIError as e:
             if e.code == 404:
                 self.die('The {0} "{1}" does not exist.'.format(what_destroy, to_destroy))
@@ -554,7 +636,7 @@ class CLI(object):
             self.user.patch(url, patch)
             deploy = True
         else:
-            self.die('Unknown sub command {0}'.format(subcmd), stderr=True)
+            self.die('Unknown sub command {0}'.format(args.subcmd), stderr=True)
         if deploy:
             self.deploy(args.application)
 
@@ -846,17 +928,19 @@ class CLI(object):
         self._yml_exists_check(args)
 
         protocol = self._selected_push_protocol(args, use_local_config=True)[1]
-        branch = self.local_config.get('push_branch') \
-                if protocol != 'rsync' else None
+        branch = None
         commit = None
         parameters = ''
 
-        if args.git or args.hg:
+        if protocol != 'rsync':
+            branch = args.branch or self.local_config.get('push_branch')
             if args.commit:
                 commit = args.commit
                 parameters = '?commit={0}'.format(args.commit)
             else:
                 branch = args.branch
+                # If we weren't passed a branch and don't have a
+                # default one, then get the current branch
                 if not branch:
                     get_local_branch = getattr(self,
                             'get_local_branch_{0}'.format(protocol), None)
@@ -967,7 +1051,7 @@ class CLI(object):
         excludes = ('*.pyc', '.git', '.hg')
         ignore_file = os.path.join(local_dir, '.dotcloudignore')
         ignore_opt = ('--exclude-from', ignore_file) if os.path.exists(ignore_file) else tuple()
-        rsync = ('rsync', '-lpthrvz', '--delete', '--safe-links') + \
+        rsync = ('rsync', '-lpthrvz', '--delete', '--safe-links', '--progress') + \
                  tuple('--exclude={0}'.format(e) for e in excludes) + \
                  ignore_opt + \
                  ('-e', ssh, local_dir,
@@ -1033,6 +1117,24 @@ class CLI(object):
             s = s.replace(c, '\\' + c)
         return s
 
+    def parse_duration(self, duration):
+        message = ''
+        if duration == '1h':
+            message = 'last 60 minutes'
+        elif duration == '6h':
+            message = 'last 6 hours'
+        elif duration == '1d':
+            message = 'last 24 hours'
+        elif duration == '1w':
+            message = 'last 7 days'
+        elif duration == '1M':
+            message = 'last 30 days'
+        else:
+            self.error('Invalid duration identifier: {0}'.format(duration))
+            self.info('Valid options are: 1h (last 60 minutes), 6h (last 6 hours), 1d (last 24 hours), 1w (last 7 days), 1M (last 30 days)')
+            self.die()
+        return message
+
     def parse_service_instance(self, service_or_instance, command):
         if '.' not in service_or_instance:
             self.die('You must specify a service and instance, e.g. "www.0"')
@@ -1043,7 +1145,7 @@ class CLI(object):
             instance_id = int(instance_id)
             if instance_id < 0:
                 raise ValueError('value should be >= 0')
-        except ValueError as e:
+        except ValueError:
             self.error('Invalid service instance identifier: {0}'.format(service_or_instance))
             self.info('Did you mean `{0} {1} -A {2} {3}` ?'.format(self.cmd, command, service_name, instance_id))
             self.die()
